@@ -1,8 +1,8 @@
 import logging
 from time import sleep
 from typing import Any, Dict, List, Optional, Union
-
 import pandas as pd
+
 from coinmetrics.api_client import CoinMetricsClient
 
 from cryptodatapy.extract.data_vendors.datavendor import DataVendor
@@ -35,6 +35,7 @@ class CoinMetrics(DataVendor):
             frequencies: List[str] = ["tick", "block", "1s", "1min", "5min", "10min", "15min", "30min",
                                       "1h", "2h", "4h", "8h", "d", "w", "m", "q"],
             base_url: Optional[str] = data_cred.coinmetrics_base_url,
+            api_endpoints: Optional[Dict[str, str]] = None,
             api_key: Optional[str] = data_cred.coinmetrics_api_key,
             max_obs_per_call: Optional[int] = None,
             rate_limit: Optional[Any] = None,
@@ -62,6 +63,9 @@ class CoinMetrics(DataVendor):
             List of available frequencies, e.g. ['tick', '1min', '5min', '10min', '15min', '30min', '1h', '2h', '4h',
         base_url: str, optional, default None
             Base url used for GET requests. If not provided, default is set to base_url stored in DataCredentials.
+        api_endpoints: dict, optional, default None
+            Dictionary with available API endpoints. If not provided, default is set to api_endpoints stored in
+            DataCredentials.
         api_key: str, optional, default None
             Api key, e.g. 'dcf13983adf7dfa79a0dfa35adf'. If not provided, default is set to
             api_key stored in DataCredentials.
@@ -71,24 +75,15 @@ class CoinMetrics(DataVendor):
         rate_limit: Any, optional, Default None
             Number of API calls made and left, by time frequency.
         """
-        DataVendor.__init__(
-            self,
-            categories,
-            exchanges,
-            indexes,
-            assets,
-            markets,
-            market_types,
-            fields,
-            frequencies,
-            base_url,
-            api_key,
-            max_obs_per_call,
-            rate_limit,
+        super().__init__(
+            categories, exchanges, indexes, assets, markets, market_types,
+            fields, frequencies, base_url, api_endpoints, api_key, max_obs_per_call, rate_limit
         )
+        self.data_req = None
+        self.data_resp = None
+        self.data = pd.DataFrame()
 
-    @staticmethod
-    def req_meta(data_type: str) -> Dict[str, Any]:
+    def req_meta(self, data_type: str) -> Dict[str, Any]:
         """
         Request metadata.
 
@@ -104,14 +99,14 @@ class CoinMetrics(DataVendor):
             Object with metadata.
         """
         try:
-            meta = getattr(client, data_type)()
+            self.data_resp = getattr(client, data_type)()
 
         except AssertionError as e:
             logging.warning(e)
             logging.warning(f"Failed to get metadata for {data_type}.")
 
         else:
-            return meta
+            return self.data_resp
 
     def get_exchanges_info(self, as_list: bool = False) -> Union[List[str], pd.DataFrame]:
         """
@@ -128,9 +123,9 @@ class CoinMetrics(DataVendor):
             List or dataframe with info on supported exchanges.
         """
         # req data
-        exch = self.req_meta(data_type='catalog_exchanges')
+        self.req_meta(data_type='catalog_exchanges')
         # wrangle data resp
-        self.exchanges = WrangleInfo(exch).cm_meta_resp(as_list=as_list, index_name='exchange')
+        self.exchanges = WrangleInfo(self.data_resp).cm_meta_resp(as_list=as_list, index_name='exchange')
 
         return self.exchanges
 
@@ -149,9 +144,9 @@ class CoinMetrics(DataVendor):
             List or dataframe with info on available indexes.
         """
         # req data
-        indexes = self.req_meta(data_type='catalog_indexes')
+        self.req_meta(data_type='catalog_indexes')
         # wrangle data resp
-        self.indexes = WrangleInfo(indexes).cm_meta_resp(as_list=as_list, index_name='ticker')
+        self.indexes = WrangleInfo(self.data_resp).cm_meta_resp(as_list=as_list, index_name='ticker')
 
         return self.indexes
 
@@ -170,33 +165,12 @@ class CoinMetrics(DataVendor):
             List or dataframe with info on available assets.
         """
         # req data
-        assets = self.req_meta(data_type='catalog_assets')
+        self.req_meta(data_type='catalog_assets')
         # wrangle data resp
-        self.assets = WrangleInfo(assets).cm_meta_resp(as_list=as_list, index_name='ticker')
+        self.assets = WrangleInfo(self.data_resp).cm_meta_resp(as_list=as_list, index_name='ticker')
 
         return self.assets
 
-    def get_inst_info(self, as_dict: bool = False) -> Union[Dict[str, List[str]], pd.DataFrame]:
-        """
-        Get institutions info.
-
-        Parameters
-        ----------
-        as_dict: bool, default False
-            Returns available institutions as dictionary.
-
-        Returns
-        -------
-        inst: dictionary or pd.DataFrame
-            Dictionary or dataframe with info on available institutions.
-        """
-        # req data
-        inst = self.req_meta(data_type='catalog_institutions')
-        # wrangle data resp
-        inst = WrangleInfo(inst).cm_inst_info(as_dict=as_dict)
-
-        return inst
-
     def get_markets_info(self, as_list: bool = False) -> Union[List[str], pd.DataFrame]:
         """
         Get markets info.
@@ -212,9 +186,9 @@ class CoinMetrics(DataVendor):
             List or dataframe with info on available markets, by exchange.
         """
         # req data
-        markets = self.req_meta(data_type='catalog_markets')
+        self.req_meta(data_type='catalog_markets')
         # wrangle data resp
-        self.markets = WrangleInfo(markets).cm_meta_resp(as_list=as_list)
+        self.markets = WrangleInfo(self.data_resp).cm_meta_resp(as_list=as_list)
 
         return self.markets
 
@@ -233,9 +207,9 @@ class CoinMetrics(DataVendor):
             List or dataframe of on-chain info.
         """
         # req data
-        onchain_fields = self.req_meta(data_type='catalog_metrics')
+        self.req_meta(data_type='catalog_metrics')
         # wrangle data resp
-        onchain_fields = WrangleInfo(onchain_fields).cm_meta_resp(as_list=as_list, index_name='fields')
+        onchain_fields = WrangleInfo(self.data_resp).cm_meta_resp(as_list=as_list, index_name='fields')
 
         return onchain_fields
 
@@ -258,14 +232,11 @@ class CoinMetrics(DataVendor):
         # req data
         ohlcv_fields = ['price_open', 'price_close', 'price_high', 'price_low', 'vwap', 'volume', 'candle_usd_volume',
                         'candle_trades_count']  # get market fields
-        inst_fields = [v for k, v in self.get_inst_info(as_dict=True).items()][0]  # inst fields
         onchain_fields = self.get_onchain_fields_info()  # get onchain fields
 
         # fields df
         if data_type == "market":
             self.fields = onchain_fields[onchain_fields.category == "Market"]
-        elif data_type == "off-chain":
-            self.fields = inst_fields
         else:
             self.fields = onchain_fields
 
@@ -275,10 +246,8 @@ class CoinMetrics(DataVendor):
                 self.fields = ohlcv_fields + list(self.fields.index)
             elif data_type == "on-chain":
                 self.fields = list(self.fields.index)
-            elif data_type == "off-chain":
-                self.fields = inst_fields
             else:
-                self.fields = ohlcv_fields + list(self.fields.index) + inst_fields
+                self.fields = ohlcv_fields + list(self.fields.index)
 
         return self.fields
 
@@ -297,15 +266,14 @@ class CoinMetrics(DataVendor):
             List of available assets for selected fields.
         """
         # convert data request parameters to Coin Metrics format
-        cm_data_req = ConvertParams(data_req).to_coinmetrics()
-        # fields param
-        fields = cm_data_req["fields"]
+        self.data_req = ConvertParams(data_req).to_coinmetrics()
 
         # fields info
         self.get_fields_info()
+
         # fields dict
         fields_dict = {}
-        for field in fields:
+        for field in self.data_req.source_fields:
             if field in self.fields.index:
                 df = self.fields.loc[field]  # get fields metadata
                 # add to dict
@@ -341,12 +309,14 @@ class CoinMetrics(DataVendor):
         if self.fields is None:
             self.get_fields_info(as_list=True)
 
-    def req_data(self, data_type: str, params: Dict[str, Union[str, int]]) -> pd.DataFrame:
+    def req_data(self, data_req: DataRequest, data_type: str, params: Dict[str, Union[str, int]]) -> pd.DataFrame:
         """
         Sends data request to Python client.
 
         Parameters
         ----------
+        data_req: DataRequest
+            Parameters of data request in CryptoDataPy format.
         data_type: str
             Data type to retrieve.
         params: dict
@@ -361,23 +331,23 @@ class CoinMetrics(DataVendor):
         url = self.base_url + data_type
 
         # data request
-        data_resp = DataRequest().get_req(url=url, params=params)
+        self.data_resp = data_req.get_req(url=url, params=params)
 
         # raise error if data is None
-        if data_resp is None:
+        if self.data_resp is None:
             raise Exception("Failed to fetch data after multiple attempts.")
         # retrieve data
         else:
             # data
-            data, next_page_url = data_resp.get('data', []), data_resp.get('next_page_url')
+            data, next_page_url = self.data_resp.get('data', []), self.data_resp.get('next_page_url')
 
             # while loop
             while next_page_url:
                 # wait to avoid exceeding rate limit
-                sleep(0.6)
+                sleep(data_req.pause)
 
                 # request next page
-                next_page_data_resp = DataRequest(pause=0.6).get_req(url=next_page_url, params=None)
+                next_page_data_resp = data_req.get_req(url=next_page_url, params=None)
                 next_page_data, next_page_url = next_page_data_resp.get('data', []), next_page_data_resp.get(
                     'next_page_url')
 
@@ -389,8 +359,7 @@ class CoinMetrics(DataVendor):
 
             return df
 
-    @staticmethod
-    def wrangle_data_resp(data_req: DataRequest, data_resp: pd.DataFrame()):
+    def wrangle_data_resp(self, data_req: DataRequest, data_resp: pd.DataFrame()) -> pd.DataFrame():
         """
         Wrangle data response.
 
@@ -433,13 +402,13 @@ class CoinMetrics(DataVendor):
             Dataframe with DatetimeIndex (level 0), ticker (level 1) and values for fields/col, in tidy data format.
         """
         # get entire data history
-        df = self.req_data(data_type, params)
+        df = self.req_data(data_req, data_type, params)
         # wrangle df
         df = self.wrangle_data_resp(data_req, df)
 
         return df
 
-    def check_tickers(self, data_req: DataRequest, data_type: str) -> List[str]:
+    def check_tickers(self, data_req: DataRequest, data_type: str) -> DataRequest:
         """
         Checks tickers for data availability.
 
@@ -457,40 +426,41 @@ class CoinMetrics(DataVendor):
             List of available tickers.
         """
         # convert params
-        cm_data_req = ConvertParams(data_req).to_coinmetrics()
-        # tickers
-        tickers = []
+        self.data_req = ConvertParams(data_req).to_coinmetrics()
 
         # check indexes
         if data_type == 'indexes':
             self.get_indexes_info(as_list=True)
             # avail tickers
-            tickers = [ticker for ticker in cm_data_req["tickers"] if ticker.upper() in self.indexes]
+            self.data_req.source_tickers = [ticker for ticker in self.data_req.source_tickers
+                                            if ticker.upper() in self.indexes]
 
         # check markets
         elif data_type == 'market_candles' or data_type == 'open_interest' or \
                 data_type == 'funding_rates' or data_type == 'trades' or data_type == 'quotes':
             self.get_assets_info(as_list=True)
             # avail tickers
-            tickers = [ticker for asset, ticker in zip(cm_data_req["tickers"], cm_data_req["mkts"]) if
-                       asset in self.assets]
+            self.data_req.source_markets = [market for ticker, market in
+                                            zip(self.data_req.source_tickers, self.data_req.source_markets)
+                                            if ticker in self.assets]
 
         # check assets
         elif data_type == 'asset_metrics':
             self.get_assets_info(as_list=True)
             # avail tickers
-            tickers = [ticker for ticker in cm_data_req["tickers"] if ticker in self.assets]
+            self.data_req.source_tickers = [ticker for ticker in self.data_req.source_tickers
+                                            if ticker in self.assets]
 
         # raise error if no tickers available
-        if len(tickers) == 0:
+        if len(self.data_req.source_tickers) == 0:
             raise ValueError(
                 f"{data_req.tickers} are not valid tickers for the requested data type."
                 f" Use get_metadata to get a list of available indexes and assets."
             )
 
-        return tickers
+        return self.data_req
 
-    def check_fields(self, data_req: DataRequest, data_type: str) -> List[str]:
+    def check_fields(self, data_req: DataRequest, data_type: str) -> DataRequest:
         """
         Checks fields for data availability.
 
@@ -508,33 +478,25 @@ class CoinMetrics(DataVendor):
             List of avaialble fields.
         """
         # convert params
-        cm_data_req = ConvertParams(data_req).to_coinmetrics()
-        # fields
-        fields = []
-
-        # check instution
-        if data_type == 'institutions':
-            # avail inst
-            inst_list = [val for key, val in self.get_inst_info(as_dict=True).items()][0]
-            fields = [field for field in cm_data_req["fields"] if field in inst_list]
+        self.data_req = ConvertParams(data_req).to_coinmetrics()
 
         # check on-chain metrics
-        elif data_type == 'asset_metrics':
+        if data_type == 'asset_metrics':
             self.get_fields_info(data_type='on-chain', as_list=True)
             # avail fields
-            fields = [field for field in cm_data_req["fields"] if field in self.fields]
+            self.data_req.source_fields = [field for field in self.data_req.source_fields
+                                           if field in self.fields]
 
         # raise error if fields is empty
-        if len(fields) == 0:
+        if len(self.data_req.source_fields) == 0:
             raise ValueError(
                 f"{data_req.fields} are not valid fields."
                 f" Use the get_fields_info or get_inst_info methods to get available source fields."
             )
 
-        return fields
+        return self.data_req
 
-    @staticmethod
-    def check_params(data_req: DataRequest, data_type: str) -> None:
+    def check_params(self, data_req: DataRequest, data_type: str) -> None:
         """
         Checks data request parameters.
 
@@ -548,27 +510,19 @@ class CoinMetrics(DataVendor):
 
         """
         # convert params
-        cm_data_req = ConvertParams(data_req).to_coinmetrics()
+        self.data_req = ConvertParams(data_req).to_coinmetrics()
 
         # indexes
         if data_type == 'indexes':
-            if cm_data_req["freq"] not in ["1h", "1d"]:
+            if self.data_req.source_freq not in ["1h", "1d"]:
                 raise ValueError(
                     f"Indexes data is only available for hourly, daily, weekly, monthly and quarterly"
                     f" frequencies. Change data request frequency and try again."
                 )
 
-        # institutions
-        elif data_type == 'institutions':
-            if cm_data_req["freq"] != "1d":
-                raise ValueError(
-                    f"Institutions data is only available for daily frequency."
-                    f" Change data request frequency and try again."
-                )
-
         # ohlcv
         elif data_type == 'market_candles':
-            if cm_data_req["freq"] not in ["1m", "1h", "1d"]:
+            if self.data_req.source_freq not in ["1m", "1h", "1d"]:
                 raise ValueError(
                     f"OHLCV data is only available for minute, hourly, daily, weekly, monthly and quarterly"
                     f" frequencies. Change data request frequency and try again."
@@ -576,7 +530,7 @@ class CoinMetrics(DataVendor):
 
         # on-chain
         elif data_type == 'asset_metrics':
-            if cm_data_req["freq"] not in ["1b", "1d"]:
+            if self.data_req.source_freq not in ["1b", "1d"]:
                 raise ValueError(
                     f"On-chain data is only available for 'block' and 'd' frequencies."
                     f" Change data request frequency and try again."
@@ -584,14 +538,15 @@ class CoinMetrics(DataVendor):
 
         # funding rate
         elif data_type == 'funding_rates':
-            if data_req.mkt_type not in ["perpetual_future", "future", "option"]:
+            if self.data_req.mkt_type not in ["perpetual_future", "future", "option"]:
                 raise ValueError(
                     f"Funding rates are only available for 'perpetual_future', 'future' and"
                     f" 'option' market types. Change 'mkt_type' in data request and try again."
                 )
 
+        # oi
         elif data_type == 'open_interest':
-            if data_req.mkt_type not in ["perpetual_future", "future", "option"]:
+            if self.data_req.mkt_type not in ["perpetual_future", "future", "option"]:
                 raise ValueError(
                     f"Open interest is only available for 'perpetual_future', 'future' and"
                     f" 'option' market types. Change 'mkt_type' in data request and try again."
@@ -599,7 +554,7 @@ class CoinMetrics(DataVendor):
 
         # trades
         elif data_type == 'trades':
-            if cm_data_req["freq"] != "raw":
+            if self.data_req.source_freq != "raw":
                 raise ValueError(
                     f"{data_type} data is only available at the 'tick' frequency."
                     f" Change data request frequency and try again."
@@ -607,7 +562,7 @@ class CoinMetrics(DataVendor):
 
         # quotes
         elif data_type == 'quotes':
-            if cm_data_req["freq"] not in ["raw", "1s", "1m", "1h", "1d"]:
+            if self.data_req.source_freq not in ["raw", "1s", "1m", "1h", "1d"]:
                 raise ValueError(
                     f"{data_type} data is only available at the 'tick', '1s', '1m', '1h' and '1d' frequencies."
                     f" Change data request frequency and try again."
@@ -630,21 +585,21 @@ class CoinMetrics(DataVendor):
             DataFrame with DatetimeIndex (level 0), tickers (level 1) and index values (cols).
         """
         # convert data request parameters to Coin Metrics format
-        cm_data_req = ConvertParams(data_req).to_coinmetrics()
+        self.data_req = ConvertParams(data_req).to_coinmetrics()
 
         # check params
         self.check_params(data_req, data_type='indexes')
 
         # check tickers
-        tickers = self.check_tickers(data_req, data_type='indexes')
-        sleep(0.6)
+        self.check_tickers(data_req, data_type='indexes')
+        sleep(self.data_req.pause)
 
         # params
         params = {
-            'indexes': ','.join(tickers),
-            'frequency': cm_data_req['freq'],
-            'start_time': cm_data_req["start_date"],
-            'end_time': cm_data_req["end_date"],
+            'indexes': ','.join(self.data_req.source_tickers),
+            'frequency': self.data_req.source_freq,
+            'start_time': self.data_req.source_start_date,
+            'end_time': self.data_req.source_end_date,
             'pretty': True,
             'page_size': 10000,
         }
@@ -656,48 +611,6 @@ class CoinMetrics(DataVendor):
 
         return df
 
-    def get_institutions(self, data_req: DataRequest) -> pd.DataFrame:
-        """
-        Get institutions data.
-
-        Parameters
-        ----------
-        data_req: DataRequest
-            Parameters of data request in CryptoDataPy format.
-
-        Returns
-        -------
-        df: pd.DataFrame
-            DataFrame with DatetimeIndex (level 0), tickers (level 1) and institution fields values (cols).
-        """
-        # convert data request parameters to Coin Metrics format
-        cm_data_req = ConvertParams(data_req).to_coinmetrics()
-
-        # check params
-        self.check_params(data_req, data_type='institutions')
-
-        # check fields
-        fields = self.check_fields(data_req, data_type='institutions')
-        sleep(0.6)
-
-        # params
-        params = {
-            'institutions': cm_data_req["inst"],
-            'metrics': ','.join(fields),
-            'frequency': cm_data_req['freq'],
-            'start_time': cm_data_req["start_date"],
-            'end_time': cm_data_req["end_date"],
-            'pretty': True,
-            'page_size': 10000,
-        }
-
-        # get tidy data
-        df = self.get_tidy_data(data_req,
-                                data_type='/timeseries/institution-metrics',
-                                params=params)
-
-        return df
-
     def get_ohlcv(self, data_req: DataRequest) -> pd.DataFrame:
         """
         Get OHLCV (candles) data.
@@ -713,21 +626,21 @@ class CoinMetrics(DataVendor):
             DataFrame with DatetimeIndex (level 0), ticker (level 1), and OHLCV values (cols).
         """
         # convert data request parameters to Coin Metrics format
-        cm_data_req = ConvertParams(data_req).to_coinmetrics()
+        self.data_req = ConvertParams(data_req).to_coinmetrics()
 
         # check freq
         self.check_params(data_req, data_type='market_candles')
 
         # check tickers
-        tickers = self.check_tickers(data_req, data_type='market_candles')
-        sleep(0.6)
+        self.check_tickers(data_req, data_type='market_candles')
+        sleep(self.data_req.pause)
 
         # params
         params = {
-            'markets': ','.join(tickers),
-            'frequency': cm_data_req['freq'],
-            'start_time': cm_data_req["start_date"],
-            'end_time': cm_data_req["end_date"],
+            'markets': ','.join(self.data_req.source_markets),
+            'frequency': self.data_req.source_freq,
+            'start_time': self.data_req.source_start_date,
+            'end_time': self.data_req.source_end_date,
             'pretty': True,
             'page_size': 10000,
         }
@@ -754,26 +667,26 @@ class CoinMetrics(DataVendor):
             DataFrame with DatetimeIndex (level 0), ticker (level 1), and on-chain values (cols).
         """
         # convert data request parameters to Coin Metrics format
-        cm_data_req = ConvertParams(data_req).to_coinmetrics()
+        self.data_req = ConvertParams(data_req).to_coinmetrics()
 
         # check params
         self.check_params(data_req, data_type='asset_metrics')
 
         # check tickers
-        tickers = self.check_tickers(data_req, data_type='asset_metrics')
-        sleep(0.6)
+        self.check_tickers(data_req, data_type='asset_metrics')
+        sleep(self.data_req.pause)
 
         # check fields
-        fields = self.check_fields(data_req, data_type='asset_metrics')
-        sleep(0.6)
+        self.check_fields(data_req, data_type='asset_metrics')
+        sleep(self.data_req.pause)
 
         # params
         params = {
-            'assets': ','.join(tickers),
-            'metrics': ','.join(fields),
-            'frequency': cm_data_req['freq'],
-            'start_time': cm_data_req["start_date"],
-            'end_time': cm_data_req["end_date"],
+            'assets': ','.join(self.data_req.source_tickers),
+            'metrics': ','.join(self.data_req.source_fields),
+            'frequency': self.data_req.source_freq,
+            'start_time': self.data_req.source_start_date,
+            'end_time': self.data_req.source_end_date,
             'pretty': True,
             'page_size': 10000,
             'ignore_forbidden_errors': True,
@@ -804,20 +717,20 @@ class CoinMetrics(DataVendor):
             DataFrame with DatetimeIndex (level 0), ticker (level 1), and open interest values (cols).
         """
         # convert data request parameters to Coin Metrics format
-        cm_data_req = ConvertParams(data_req).to_coinmetrics()
+        self.data_req = ConvertParams(data_req).to_coinmetrics()
 
         # check params
         self.check_params(data_req, data_type='open_interest')
 
         # check tickers
-        tickers = self.check_tickers(data_req, data_type='open_interest')
-        sleep(0.6)
+        self.check_tickers(data_req, data_type='open_interest')
+        sleep(self.data_req.pause)
 
         # params
         params = {
-            'markets': ','.join(tickers),
-            'start_time': cm_data_req["start_date"],
-            'end_time': cm_data_req["end_date"],
+            'markets': ','.join(self.data_req.source_markets),
+            'start_time': self.data_req.source_start_date,
+            'end_time': self.data_req.source_end_date,
             'pretty': True,
             'page_size': 10000,
         }
@@ -845,20 +758,20 @@ class CoinMetrics(DataVendor):
             DataFrame with DatetimeIndex (level 0), ticker (level 1), and funding rates values (cols).
         """
         # convert data request parameters to Coin Metrics format
-        cm_data_req = ConvertParams(data_req).to_coinmetrics()
+        self.data_req = ConvertParams(data_req).to_coinmetrics()
 
         # check params
         self.check_params(data_req, data_type='funding_rates')
 
         # check tickers
-        tickers = self.check_tickers(data_req, data_type='funding_rates')
-        sleep(0.6)
+        self.check_tickers(data_req, data_type='funding_rates')
+        sleep(self.data_req.pause)
 
         # params
         params = {
-            'markets': ','.join(tickers),
-            'start_time': cm_data_req["start_date"],
-            'end_time': cm_data_req["end_date"],
+            'markets': ','.join(self.data_req.source_markets),
+            'start_time': self.data_req.source_start_date,
+            'end_time': self.data_req.source_end_date,
             'pretty': True,
             'page_size': 10000,
         }
@@ -886,20 +799,20 @@ class CoinMetrics(DataVendor):
             DataFrame with DatetimeIndex (level 0), ticker (level 1), and bid/ask price and size values (cols).
         """
         # convert data request parameters to Coin Metrics format
-        cm_data_req = ConvertParams(data_req).to_coinmetrics()
+        self.data_req = ConvertParams(data_req).to_coinmetrics()
 
         # check params
         self.check_params(data_req, data_type='trades')
 
         # check tickers
-        tickers = self.check_tickers(data_req, data_type='trades')
-        sleep(0.6)
+        self.check_tickers(data_req, data_type='trades')
+        sleep(self.data_req.pause)
 
         # params
         params = {
-            'markets': ','.join(tickers),
-            'start_time': cm_data_req["start_date"],
-            'end_time': cm_data_req["end_date"],
+            'markets': ','.join(self.data_req.source_markets),
+            'start_time': self.data_req.source_start_date,
+            'end_time': self.data_req.source_end_date,
             'pretty': True,
             'page_size': 10000,
         }
@@ -927,21 +840,21 @@ class CoinMetrics(DataVendor):
             DataFrame with DatetimeIndex (level 0), ticker (level 1), and bid/ask price and size values (cols).
         """
         # convert data request parameters to Coin Metrics format
-        cm_data_req = ConvertParams(data_req).to_coinmetrics()
+        self.data_req = ConvertParams(data_req).to_coinmetrics()
 
         # check params
         self.check_params(data_req, data_type='quotes')
 
         # check tickers
-        tickers = self.check_tickers(data_req, data_type='quotes')
-        sleep(0.6)
+        self.check_tickers(data_req, data_type='quotes')
+        sleep(self.data_req.pause)
 
         # params
         params = {
-            'markets': ','.join(tickers),
-            'granularity': cm_data_req['freq'],
-            'start_time': cm_data_req["start_date"],
-            'end_time': cm_data_req["end_date"],
+            'markets': ','.join(self.data_req.source_markets),
+            'granularity': self.data_req.source_freq,
+            'start_time': self.data_req.source_start_date,
+            'end_time': self.data_req.source_end_date,
             'pretty': True,
             'page_size': 10000,
         }
@@ -970,57 +883,55 @@ class CoinMetrics(DataVendor):
             fields (cols), in tidy format.
         """
         # convert data request parameters to Coin Metrics format
-        cm_data_req = ConvertParams(data_req).to_coinmetrics()
+        self.data_req = ConvertParams(data_req).to_coinmetrics()
 
         # check if fields available
         self.get_fields_info(as_list=True)
-        sleep(0.6)
-        # if not all([field in self.fields for field in cm_data_req["fields"]]):
-        #     raise ValueError(
-        #         "Some selected fields are not available. Check available fields with"
-        #         " get_fields_info method and try again."
-        #     )
+        sleep(self.data_req.pause)
+
+        if not all([field in self.fields for field in self.data_req.source_fields]):
+            raise ValueError(
+                "Some selected fields are not available. Check available fields with"
+                " get_fields_info method and try again."
+            )
 
         # field lists
         ohlcv_list = ['price_open', 'price_close', 'price_high', 'price_low', 'vwap', 'volume',
                       'candle_usd_volume', 'candle_trades_count']
         oc_list = [field for field in self.fields if field not in ohlcv_list]
 
-        # empty df
-        df = pd.DataFrame()
-
         # get indexes data
         self.get_indexes_info(as_list=True)
-        sleep(0.6)
-        if any([ticker.upper() in self.indexes for ticker in cm_data_req["tickers"]]) and any(
-                [field in ohlcv_list for field in cm_data_req["fields"]]
+        sleep(self.data_req.pause)
+        if any([ticker.upper() in self.indexes for ticker in self.data_req.source_tickers]) and any(
+                [field in ohlcv_list for field in self.data_req.source_fields]
         ):
-            df0 = self.get_indexes(data_req)
-            df = pd.concat([df, df0])
+            df = self.get_indexes(data_req)
+            self.data = pd.concat([self.data, df])
 
         # get OHLCV data
         self.get_assets_info(as_list=True)
-        sleep(0.6)
-        if any([ticker in self.assets for ticker in cm_data_req["tickers"]]) and any(
-                [field in ohlcv_list for field in cm_data_req["fields"]]
+        sleep(self.data_req.pause)
+        if any([ticker in self.assets for ticker in self.data_req.source_tickers]) and any(
+                [field in ohlcv_list for field in self.data_req.source_fields]
         ):
             df1 = self.get_ohlcv(data_req)
-            df = pd.concat([df, df1])
+            self.data = pd.concat([self.data, df1])
 
         # get on-chain data
-        if any([ticker in self.assets for ticker in cm_data_req["tickers"]]) and any(
-                [field in oc_list for field in cm_data_req["fields"]]
+        if any([ticker in self.assets for ticker in self.data_req.source_tickers]) and any(
+                [field in oc_list for field in self.data_req.source_fields]
         ):
             df2 = self.get_onchain(data_req)
-            df = pd.concat([df, df2], axis=1)
+            self.data = pd.concat([self.data, df2], axis=1)
 
         # check if df empty
-        if df.empty:
+        if self.data.empty:
             raise Exception("No data returned."
                             " Check data request parameters and try again.")
 
         # filter df for desired fields and sort index by date
-        fields = [field for field in data_req.fields if field in df.columns]
-        df = df.loc[:, fields]
+        fields = [field for field in data_req.fields if field in self.data.columns]
+        self.data = self.data.loc[:, fields].sort_index()
 
-        return df.sort_index()
+        return self.data
