@@ -2,6 +2,7 @@ import logging
 from datetime import datetime, timedelta
 from importlib import resources
 from typing import Dict, List, Union
+import re
 
 import pandas as pd
 
@@ -24,236 +25,189 @@ class ConvertParams:
         """
         self.data_req = data_req
 
-    def to_cryptocompare(self) -> Dict[str, Union[list, str, int, float, None]]:
+    def to_cryptocompare(self) -> DataRequest:
         """
         Convert tickers from CryptoDataPy to CryptoCompare format.
         """
-        # convert tickers
-        if self.data_req.source_tickers is not None:
-            tickers = self.data_req.source_tickers
-            self.data_req.tickers = self.data_req.source_tickers
-        else:
-            tickers = [ticker.upper() for ticker in self.data_req.tickers]
-        # convert freq
-        if self.data_req.source_freq is not None:
-            freq = self.data_req.source_freq
-            self.data_req.freq = self.data_req.source_freq
-        else:
-            if self.data_req.freq[-3:] == "min":
-                freq = "histominute"
+        # tickers
+        if self.data_req.source_tickers is None:
+            self.data_req.source_tickers = [ticker.upper() for ticker in self.data_req.tickers]
+
+        # freq
+        if self.data_req.source_freq is None:
+            if self.data_req.freq is None:
+                self.data_req.source_freq = "histoday"
+            elif self.data_req.freq[-3:] == "min":
+                self.data_req.source_freq = "histominute"
             elif self.data_req.freq[-1] == "h":
-                freq = "histohour"
+                self.data_req.source_freq = "histohour"
             else:
-                freq = "histoday"
-        # convert quote ccy
+                self.data_req.source_freq = "histoday"
+
+        # quote ccy
         if self.data_req.quote_ccy is None:
-            quote_ccy = "USD"
+            self.data_req.quote_ccy = "USD"
         else:
-            quote_ccy = self.data_req.quote_ccy.upper()
+            self.data_req.quote_ccy = self.data_req.quote_ccy.upper()
+
         # convert exch
         if self.data_req.exch is None:
-            exch = "CCCAGG"
+            self.data_req.exch = "CCCAGG"
         else:
-            exch = self.data_req.exch
-        # convert start date
+            self.data_req.exch = self.data_req.exch.lower()
+
+        # start date
         if self.data_req.freq[-3:] == "min":  # limit to higher frequency data responses
-            start_date = int((datetime.now() - timedelta(days=7)).timestamp())
-        # no start date
+            self.data_req.source_start_date = int((datetime.now() - timedelta(days=7)).timestamp())
         elif self.data_req.start_date is None:
-            start_date = int(pd.Timestamp("2009-01-03 00:00:00").timestamp())
+            self.data_req.source_start_date = int(pd.Timestamp("2009-01-03 00:00:00").timestamp())
         else:
-            start_date = int(pd.Timestamp(self.data_req.start_date).timestamp())
+            self.data_req.source_start_date = int(pd.Timestamp(self.data_req.start_date).timestamp())
+
         # convert end date
         if self.data_req.end_date is None:
-            end_date = int(pd.Timestamp.utcnow().timestamp())
+            self.data_req.source_end_date = int(pd.Timestamp.utcnow().timestamp())
         else:
-            end_date = int(pd.Timestamp(self.data_req.end_date).timestamp())
+            self.data_req.source_end_date = int(pd.Timestamp(self.data_req.end_date).timestamp())
+
         # fields
-        if self.data_req.source_fields is not None:
-            fields = self.data_req.source_fields
-            self.data_req.fields = self.data_req.source_fields
-        else:
-            fields = self.convert_fields(data_source='cryptocompare')
+        if self.data_req.source_fields is None:
+            self.data_req.source_fields = self.convert_fields(data_source='cryptocompare')
+
         # tz
         if self.data_req.tz is None:
-            tz = "UTC"
-        else:
-            tz = self.data_req.tz
+            self.data_req.tz = "UTC"
 
-        return {
-            "tickers": tickers,
-            "freq": freq,
-            "quote_ccy": quote_ccy,
-            "exch": exch,
-            "ctys": None,
-            "mkt_type": self.data_req.mkt_type,
-            "mkts": None,
-            "start_date": start_date,
-            "end_date": end_date,
-            "fields": fields,
-            "tz": tz,
-            "inst": None,
-            "cat": 'crypto',
-            "trials": self.data_req.trials,
-            "pause": self.data_req.pause,
-            "source_tickers": self.data_req.source_tickers,
-            "source_freq": self.data_req.source_freq,
-            "source_fields": self.data_req.source_fields,
-        }
+        return self.data_req
 
-    def to_coinmetrics(self) -> Dict[str, Union[list, str, int, float, None]]:
+    def to_coinmetrics(self) -> DataRequest:
         """
         Convert tickers from CryptoDataPy to CoinMetrics format.
         """
-        # convert tickers
-        if self.data_req.source_tickers is not None:
-            tickers = self.data_req.source_tickers
-            self.data_req.tickers = self.data_req.source_tickers
-        else:
-            tickers = [ticker.lower() for ticker in self.data_req.tickers]
-        # convert freq
-        if self.data_req.source_freq is not None:
-            freq = self.data_req.source_freq
-            self.data_req.freq = self.data_req.source_freq
-        else:
+        # tickers
+        if self.data_req.source_tickers is None:
+            self.data_req.source_tickers = [ticker.lower() for ticker in self.data_req.tickers]
+
+        # freq
+        if self.data_req.source_freq is None:
             if self.data_req.freq is None:
-                freq = "1d"
+                self.data_req.source_freq = "1d"
             elif self.data_req.freq == "block":
-                freq = "1b"
+                self.data_req.source_freq = "1b"
             elif self.data_req.freq == "tick":
-                freq = "raw"
+                self.data_req.source_freq = "raw"
             elif self.data_req.freq[-1] == "s":
-                freq = "1s"
+                self.data_req.source_freq = "1s"
             elif self.data_req.freq[-3:] == "min":
-                freq = "1m"
+                self.data_req.source_freq = "1m"
             elif self.data_req.freq[-1] == "h":
-                freq = "1h"
+                self.data_req.source_freq = "1h"
             else:
-                freq = "1d"
-        # convert quote ccy
+                self.data_req.source_freq = "1d"
+
+        # quote ccy
         if self.data_req.quote_ccy is None:
-            quote_ccy = "usdt"
+            self.data_req.quote_ccy = "usdt"
         else:
-            quote_ccy = self.data_req.quote_ccy.lower()
-        # convert inst
-        if self.data_req.inst is None:
-            inst = "grayscale"
-        else:
-            inst = self.data_req.inst.lower()
-        # convert to exch
+            self.data_req.quote_ccy = self.data_req.quote_ccy.lower()
+
+        # exchange
         if self.data_req.exch is None:
-            exch = "binance"
+            self.data_req.exch = "binance"
         else:
-            exch = self.data_req.exch.lower()
+            self.data_req.exch = self.data_req.exch.lower()
+
         # fields
-        if self.data_req.source_fields is not None:
-            fields = self.data_req.source_fields
-            self.data_req.fields = self.data_req.source_fields
-        else:
-            fields = self.convert_fields(data_source='coinmetrics')
+        if self.data_req.source_fields is None:
+            self.data_req.source_fields = self.convert_fields(data_source='coinmetrics')
+
         # convert tz
         if self.data_req.tz is None:
-            tz = "UTC"
+            self.data_req.tz = "UTC"
         else:
-            tz = self.data_req.tz
-        # convert tickers to markets
+            self.data_req.tz = self.data_req.tz
+
+        # markets
         mkts_list = []
 
-        if self.data_req.source_tickers is not None:
-            mkts_list = self.data_req.source_tickers
-            self.data_req.tickers = self.data_req.source_tickers
-
-        for ticker in self.data_req.tickers:
-            if self.data_req.mkt_type == "spot":
-                mkts_list.append(
-                    exch
-                    + "-"
-                    + ticker.lower()
-                    + "-"
-                    + quote_ccy.lower()
-                    + "-"
-                    + self.data_req.mkt_type.lower()
-                )
-            elif self.data_req.mkt_type == "perpetual_future":
-                if exch == "binance" or exch == "bybit" or exch == "bitmex":
+        if self.data_req.source_markets is None:
+            # loop through tickers
+            for ticker in self.data_req.tickers:
+                if self.data_req.mkt_type == "spot":
                     mkts_list.append(
-                        exch
+                        self.data_req.exch
                         + "-"
-                        + ticker.upper()
-                        + quote_ccy.upper()
+                        + ticker.lower()
                         + "-"
-                        + "future"
+                        + self.data_req.quote_ccy.lower()
+                        + "-"
+                        + self.data_req.mkt_type.lower()
                     )
-                elif exch == "ftx":
-                    mkts_list.append(
-                        exch + "-" + ticker.upper() + "-" + "PERP" + "-" + "future"
+                elif self.data_req.mkt_type == "perpetual_future":
+                    if (self.data_req.exch == "binance" or self.data_req.exch == "bybit" or
+                            self.data_req.exch == "bitmex"):
+                        mkts_list.append(
+                            self.data_req.exch
+                            + "-"
+                            + ticker.upper()
+                            + self.data_req.quote_ccy.upper()
+                            + "-"
+                            + "future"
                         )
-                elif exch == "okex":
-                    mkts_list.append(
-                        exch
-                        + "-"
-                        + ticker.upper()
-                        + "-"
-                        + quote_ccy.upper()
-                        + "-"
-                        + "SWAP"
-                        + "-"
-                        + "future"
-                    )
-                elif exch == "huobi":
-                    mkts_list.append(
-                        exch
-                        + "-"
-                        + ticker.upper()
-                        + "-"
-                        + quote_ccy.upper()
-                        + "_"
-                        + "SWAP"
-                        + "-"
-                        + "future"
-                    )
-                elif exch == "hitbtc":
-                    mkts_list.append(
-                        exch
-                        + "-"
-                        + ticker.upper()
-                        + quote_ccy.upper()
-                        + "_"
-                        + "PERP"
-                        + "-"
-                        + "future"
-                    )
+                    elif self.data_req.exch == "ftx":
+                        mkts_list.append(
+                            self.data_req.exch + "-" + ticker.upper() + "-" + "PERP" + "-" + "future"
+                            )
+                    elif self.data_req.exch == "okex":
+                        mkts_list.append(
+                            self.data_req.exch
+                            + "-"
+                            + ticker.upper()
+                            + "-"
+                            + self.data_req.quote_ccy.upper()
+                            + "-"
+                            + "SWAP"
+                            + "-"
+                            + "future"
+                        )
+                    elif self.data_req.exch == "huobi":
+                        mkts_list.append(
+                            self.data_req.exch
+                            + "-"
+                            + ticker.upper()
+                            + "-"
+                            + self.data_req.quote_ccy.upper()
+                            + "_"
+                            + "SWAP"
+                            + "-"
+                            + "future"
+                        )
+                    elif self.data_req.exch == "hitbtc":
+                        mkts_list.append(
+                            self.data_req.exch
+                            + "-"
+                            + ticker.upper()
+                            + self.data_req.quote_ccy.upper()
+                            + "_"
+                            + "PERP"
+                            + "-"
+                            + "future"
+                        )
+            self.data_req.source_markets = mkts_list
+
         # start date
         if self.data_req.start_date is not None:
-            start_date = self.data_req.start_date.strftime('%Y-%m-%d')
+            self.data_req.source_start_date = self.data_req.start_date.strftime('%Y-%m-%d')
         else:
-            start_date = None
+            self.data_req.source_start_date = None
+
         # end date
         if self.data_req.end_date is not None:
-            end_date = self.data_req.end_date.strftime('%Y-%m-%d')
+            self.data_req.source_end_date = self.data_req.end_date.strftime('%Y-%m-%d')
         else:
-            end_date = None
+            self.data_req.source_end_date = None
 
-        return {
-            "tickers": tickers,
-            "freq": freq,
-            "quote_ccy": quote_ccy,
-            "exch": exch,
-            "ctys": None,
-            "mkt_type": self.data_req.mkt_type,
-            "mkts": mkts_list,
-            "start_date": start_date,
-            "end_date": end_date,
-            "fields": fields,
-            "tz": tz,
-            "inst": inst,
-            "cat": 'crypto',
-            "trials": self.data_req.trials,
-            "pause": self.data_req.pause,
-            "source_tickers": self.data_req.source_tickers,
-            "source_freq": self.data_req.source_freq,
-            "source_fields": self.data_req.source_fields,
-        }
+        return self.data_req
 
     def to_glassnode(self) -> Dict[str, Union[list, str, int, float, None]]:
         """
@@ -339,34 +293,43 @@ class ConvertParams:
             "source_fields": self.data_req.source_fields,
         }
 
-    def to_tiingo(self) -> Dict[str, Union[list, str, int, float, datetime, None]]:
+    def to_tiingo(self) -> DataRequest:
         """
         Convert tickers from CryptoDataPy to Tiingo format.
         """
-        # convert tickers
-        if self.data_req.source_tickers is not None:
-            tickers = self.data_req.source_tickers
-            self.data_req.tickers = self.data_req.source_tickers
-        else:
-            tickers = [ticker.lower() for ticker in self.data_req.tickers]
-        # convert freq
-        if self.data_req.source_freq is not None:
-            freq = self.data_req.source_freq
-            self.data_req.freq = self.data_req.source_freq
-        else:
+        # tickers
+        with resources.path("cryptodatapy.conf", "tickers.csv") as f:
+            tickers_path = f
+        tickers_df = pd.read_csv(tickers_path, index_col=0, encoding="latin1")
+
+        if self.data_req.source_tickers is None and self.data_req.cat == 'eqty':
+            self.data_req.source_tickers = []
+            for ticker in self.data_req.tickers:
+                try:
+                    self.data_req.source_tickers.append(tickers_df.loc[ticker, "tiingo_id"])
+                except KeyError:
+                    logging.warning(
+                        f"{ticker} not found for Tiingo source. Check tickers in"
+                        f" data catalog or try using source_tickers parameter."
+                    )
+
+        # freq
+        if self.data_req.source_freq is None:
             if self.data_req.freq is None:
-                freq = "1day"
+                self.data_req.source_freq = "1day"
             elif self.data_req.freq[-3:] == "min":
-                freq = self.data_req.freq
+                self.data_req.source_freq = self.data_req.freq
             elif self.data_req.freq[-1] == "h":
-                freq = "1hour"
+                self.data_req.source_freq = "1hour"
             else:
-                freq = "1day"
-        # convert quote ccy
+                self.data_req.source_freq = "1day"
+
+        # quote ccy
         if self.data_req.quote_ccy is None:
-            quote_ccy = "usd"
+            self.data_req.quote_ccy = "usd"
         else:
-            quote_ccy = self.data_req.quote_ccy.lower()
+            self.data_req.quote_ccy = self.data_req.quote_ccy.lower()
+
         # convert exch
         if (
             self.data_req.exch is None
@@ -374,64 +337,40 @@ class ConvertParams:
             and self.data_req.freq
             in ["1min", "5min", "10min", "15min", "30min", "1h", "2h", "4h", "8h"]
         ):
-            exch = "iex"
+            self.data_req.exch = "iex"
+
+        # markets
+        if self.data_req.source_markets is None:
+            if self.data_req.cat == 'fx':
+                fx_list = self.convert_fx_tickers(quote_ccy=self.data_req.quote_ccy)
+                self.data_req.source_markets = [ticker.lower().replace("/", "") for ticker in fx_list]
+            elif self.data_req.cat == 'crypto':
+                self.data_req.source_markets = [ticker.lower() +
+                                                self.data_req.quote_ccy for ticker in self.data_req.tickers]
+
+        # start date
+        if self.data_req.start_date is None:
+            self.data_req.source_start_date = '1990-01-01'
         else:
-            exch = self.data_req.exch
-        # convert tickers to mkts
-        mkts_list = []
-        if self.data_req.source_tickers is not None:
-            mkts_list = self.data_req.source_tickers
-            self.data_req.tickers = self.data_req.source_tickers
-        else:
-            if self.data_req.cat == "fx":
-                fx_list = self.convert_fx_tickers(quote_ccy=quote_ccy)
-                mkts_list = [ticker.lower().replace("/", "") for ticker in fx_list]
-            elif self.data_req.cat == "crypto":
-                mkts_list = [
-                    ticker.lower() + quote_ccy for ticker in self.data_req.tickers
-                ]
-        # convert start date
-        if self.data_req.start_date is None and self.data_req.cat == 'crypto':
-            start_date = datetime(2010, 1, 1, 0, 0)
-        else:
-            start_date = self.data_req.start_date
-        # convert end date
+            self.data_req.source_start_date = self.data_req.start_date
+
+        # end date
         if self.data_req.end_date is None:
-            end_date = pd.Timestamp.utcnow()
+            self.data_req.source_end_date = str(pd.Timestamp.utcnow().date())
         else:
-            end_date = self.data_req.end_date
-        # convert fields
-        if self.data_req.source_fields is not None:
-            fields = self.data_req.source_fields
-            self.data_req.fields = self.data_req.source_fields
-        else:
-            fields = self.convert_fields(data_source='tiingo')
+            self.data_req.source_end_date = self.data_req.end_date
+
+        # fields
+        if self.data_req.source_fields is None:
+            self.data_req.source_fields = self.convert_fields(data_source='tiingo')
+
         # tz
         if self.data_req.cat == 'eqty' or self.data_req.cat == 'fx':
-            tz = "America/New_York"
+            self.data_req.tz = "America/New_York"
         else:
-            tz = "UTC"
+            self.data_req.tz = "UTC"
 
-        return {
-            "tickers": tickers,
-            "freq": freq,
-            "quote_ccy": quote_ccy,
-            "exch": exch,
-            "ctys": None,
-            "mkt_type": self.data_req.mkt_type,
-            "mkts": mkts_list,
-            "start_date": start_date,
-            "end_date": end_date,
-            "fields": fields,
-            "tz": tz,
-            "inst": None,
-            "cat": self.data_req.cat,
-            "trials": self.data_req.trials,
-            "pause": self.data_req.pause,
-            "source_tickers": self.data_req.source_tickers,
-            "source_freq": self.data_req.source_freq,
-            "source_fields": self.data_req.source_fields,
-        }
+        return self.data_req
 
     def to_ccxt(self) -> DataRequest:
         """
@@ -729,7 +668,7 @@ class ConvertParams:
 
     def to_wb(self) -> Dict[str, Union[list, str, int, float, datetime, None]]:
         """
-        Convert tickers from CryptoDataPy to Yahoo Finance format.
+        Convert tickers from CryptoDataPy to World Bank format.
         """
         # tickers
         with resources.path("cryptodatapy.conf", "tickers.csv") as f:
@@ -842,6 +781,72 @@ class ConvertParams:
 
         return self.data_req
 
+    def to_alphavantage(self) -> DataRequest:
+        """
+        Convert tickers from CryptoDataPy to Alpha Vantage format.
+        """
+        # tickers
+        if self.data_req.source_tickers is None:
+            self.data_req.source_tickers = [ticker.upper() for ticker in self.data_req.tickers]
+
+        # convert quote ccy
+        if self.data_req.quote_ccy is None:
+            self.data_req.quote_ccy = "USD"
+        else:
+            self.data_req.quote_ccy = self.data_req.quote_ccy.upper()
+
+        # start date
+        self.data_req.source_start_date = self.data_req.start_date
+
+        # end date
+        self.data_req.source_end_date = self.data_req.end_date
+
+        # fields
+        if self.data_req.source_fields is None:
+            self.data_req.source_fields = self.data_req.fields
+
+        # tz
+        if self.data_req.tz is None:
+            self.data_req.tz = "America/New_York"
+
+        # freq
+        if self.data_req.cat == 'eqty':
+
+            # freq
+            if self.data_req.source_freq is None:
+                self.data_req.source_freq = 'av-daily'
+            elif self.data_req.freq in ['1min', '5min', '15min', '30min', '1h', '2h', '4h', '6h', '8h']:
+                self.data_req.source_freq = 'av-intraday'
+            elif self.data_req.freq == 'd':
+                self.data_req.source_freq = 'av-daily'
+            elif self.data_req.freq == 'w':
+                self.data_req.source_freq = 'av-weekly'
+            elif self.data_req.freq == 'm':
+                self.data_req.source_freq = 'av-monthly'
+            else:
+                self.data_req.source_freq = 'av-daily'
+
+            # adjusted prices
+            if any(col.endswith('_adj') for col in self.data_req.fields) and self.data_req.freq in ['d', 'w', 'm']:
+                self.data_req.source_freq = self.data_req.source_freq + '-adjusted'
+
+            # markets
+            if self.data_req.source_markets is None:
+                self.data_req.source_markets = self.data_req.source_tickers
+
+        elif self.data_req.cat == 'fx':
+
+            # freq
+            if self.data_req.source_freq is None:
+                self.data_req.source_freq = 'av-forex-daily'
+
+            # markets
+            if self.data_req.source_markets is None:
+                self.data_req.source_markets = [ticker + '/' + self.data_req.quote_ccy
+                                                for ticker in self.data_req.tickers]
+
+        return self.data_req
+
     def to_famafrench(self) -> DataRequest:
         """
         Convert tickers from CryptoDataPy to Fama-French format.
@@ -880,6 +885,84 @@ class ConvertParams:
 
         return self.data_req
 
+    def to_polygon(self) -> DataRequest:
+        """
+        Convert tickers from CryptoDataPy to Polygon format.
+        """
+        # tickers
+        with resources.path("cryptodatapy.conf", "tickers.csv") as f:
+            tickers_path = f
+        tickers_df = pd.read_csv(tickers_path, index_col=0, encoding="latin1")
+
+        if self.data_req.source_tickers is None and self.data_req.cat == 'eqty':
+            self.data_req.source_tickers = []
+            for ticker in self.data_req.tickers:
+                try:
+                    self.data_req.source_tickers.append(tickers_df.loc[ticker, "polygon_id"])
+                except KeyError:
+                    logging.warning(
+                        f"{ticker} not found for Polygon source. Check tickers in"
+                        f" data catalog or try using source_tickers parameter."
+                    )
+
+        # freq
+        if self.data_req.source_freq is None:
+            if self.data_req.freq is None:
+                self.data_req.source_freq = "day"
+            elif self.data_req.freq[-1] == "s":
+                self.data_req.source_freq = "second"
+            elif self.data_req.freq[-3:] == "min":
+                self.data_req.source_freq = "minute"
+            elif self.data_req.freq[-1] == "h":
+                self.data_req.source_freq = "hour"
+            elif self.data_req.freq == "w":
+                self.data_req.source_freq = "week"
+            elif self.data_req.freq == "m":
+                self.data_req.source_freq = "month"
+            elif self.data_req.freq == "q":
+                self.data_req.source_freq = "quarter"
+            elif self.data_req.freq == "y":
+                self.data_req.source_freq = "year"
+            else:
+                self.data_req.source_freq = "day"
+
+        # quote ccy
+        if self.data_req.quote_ccy is None:
+            self.data_req.quote_ccy = "usd"
+        else:
+            self.data_req.quote_ccy = self.data_req.quote_ccy.lower()
+
+        # markets
+        if self.data_req.source_markets is None:
+            if self.data_req.cat == 'fx':
+                self.data_req.source_markets = [ticker.upper() + self.data_req.quote_ccy.upper()
+                           for ticker in self.data_req.tickers]
+
+        # start date
+        if self.data_req.start_date is None:
+            two_years_ago = pd.Timestamp.today() - pd.DateOffset(years=2)
+            self.data_req.source_start_date = two_years_ago.strftime("%Y-%m-%d")
+        else:
+            self.data_req.source_start_date = self.data_req.start_date
+
+        # end date
+        if self.data_req.end_date is None:
+            self.data_req.source_end_date = str(pd.Timestamp.utcnow().date())
+        else:
+            self.data_req.source_end_date = self.data_req.end_date
+
+        # fields
+        if self.data_req.source_fields is None:
+            self.data_req.source_fields = self.convert_fields(data_source='polygon')
+
+        # tz
+        if self.data_req.cat == 'eqty' or self.data_req.cat == 'fx':
+            self.data_req.tz = "America/New_York"
+        else:
+            self.data_req.tz = "UTC"
+
+        return self.data_req
+
     def to_aqr(self) -> Dict[str, Union[list, str, int, dict, float, datetime, None]]:
         """
         Convert tickers from CryptoDataPy to AQR format.
@@ -973,79 +1056,6 @@ class ConvertParams:
             "source_fields": self.data_req.source_fields,
         }
 
-    def convert_fx_tickers(self, quote_ccy: str) -> List[str]:
-        """
-        Converts base and quote currency tickers to fx pairs following fx quoting convention.
-
-        Parameters
-        ---------
-        quote_ccy: str
-            Quote currency
-
-        Returns
-        -------
-        quote_ccy: str
-            Quote currency.
-        """
-        mkts = []  # fx pairs list
-        # fx groups
-        base_ccys = ["EUR", "GBP", "AUD", "NZD"]
-        # g10_fx = ['USD', 'EUR', 'GBP', 'JPY', 'CHF', 'CAD', 'AUD', 'NZD', 'NOK', 'SEK']
-        # dm_fx = ['USD', 'EUR', 'GBP', 'JPY', 'CHF', 'CAD', 'AUD', 'NZD', 'NOK', 'SEK', 'SGD', 'ILS', 'HKD', ]
-        # em_fx = ['ARS', 'BRL', 'CHN', 'CLP', 'CNY', 'COP', 'IDR', 'INR', 'KRW', 'MYR', 'MXN', 'PEN', 'PHP', 'RUB',
-        #          'TRY', 'TWD', 'ZAR']
-
-        for ticker in self.data_req.tickers:
-            if ticker.upper() in base_ccys and quote_ccy.upper() == "USD":
-                mkts.append(ticker.upper() + "/" + quote_ccy.upper())
-            elif quote_ccy.upper() == "USD":
-                mkts.append(quote_ccy.upper() + "/" + ticker.upper())
-            else:
-                mkts.append(ticker.upper() + "/" + quote_ccy.upper())
-
-        return mkts
-
-    def convert_fields(self, data_source: str) -> List[str]:
-        """
-        Converts fields from CryptoDataPy to data source format.
-
-        Parameters
-        ---------
-        data_source: str
-            Name of data source for fields conversions.
-
-        Returns
-        -------
-        fields_list: list
-            List of fields in data source format.
-
-        """
-        # x fields
-        with resources.path("cryptodatapy.conf", "fields.csv") as f:
-            fields_dict_path = f
-        fields_df, fields_list = (
-            pd.read_csv(fields_dict_path, index_col=0, encoding="latin1"),
-            [],
-        )
-
-        # when source fields already provided in data req
-        if self.data_req.source_fields is not None:
-            fields_list = self.data_req.source_fields
-
-        # convert to source format
-        else:
-            for field in self.data_req.fields:
-                try:
-                    fields_list.append(fields_df.loc[field, data_source + "_id"])
-                except KeyError as e:
-                    logging.warning(e)
-                    logging.warning(
-                        f"Id for {field} could not be found in the data catalog."
-                        f" Try using source field ids."
-                    )
-
-        return fields_list
-
     def to_dydx_dict(self) -> Dict[str, Union[list, str, int, float, None]]:
         """
         Convert parameters from CryptoDataPy to dYdX format.
@@ -1106,23 +1116,23 @@ class ConvertParams:
                 else:
                     logging.warning(f"Field {field} not available in dYdX API")
             return {
-            "tickers": tickers,  # List of market tickers
-            "freq": freq,        # Converted frequency
-            "quote_ccy": self.data_req.quote_ccy,
-            "exch": "dydx",
-            "mkt_type": self.data_req.mkt_type,
-            "mkts": markets,     # Market identifiers
-            "start_date": self.data_req.start_date,
-            "end_date": self.data_req.end_date,
-            "fields": fields,    # Converted field names
-            "tz": self.data_req.tz or "UTC",
-            "cat": "crypto",
-            "trials": self.data_req.trials,
-            "pause": self.data_req.pause,
-            "source_tickers": self.data_req.source_tickers,
-            "source_freq": self.data_req.source_freq,
-            "source_fields": self.data_req.source_fields,
-        }
+                "tickers": tickers,  # List of market tickers
+                "freq": freq,        # Converted frequency
+                "quote_ccy": self.data_req.quote_ccy,
+                "exch": "dydx",
+                "mkt_type": self.data_req.mkt_type,
+                "mkts": markets,     # Market identifiers
+                "start_date": self.data_req.start_date,
+                "end_date": self.data_req.end_date,
+                "fields": fields,    # Converted field names
+                "tz": self.data_req.tz or "UTC",
+                "cat": "crypto",
+                "trials": self.data_req.trials,
+                "pause": self.data_req.pause,
+                "source_tickers": self.data_req.source_tickers,
+                "source_freq": self.data_req.source_freq,
+                "source_fields": self.data_req.source_fields
+            }
 
     def to_dydx(self) -> DataRequest:
 
@@ -1162,4 +1172,79 @@ class ConvertParams:
                 'funding_rate': 'nextFundingRate',
                 'oi': 'openInterest'
             }
-        return self.data_req
\ No newline at end of file
+        return self.data_req
+
+    def convert_fx_tickers(self, quote_ccy: str) -> List[str]:
+        """
+        Converts base and quote currency tickers to fx pairs following fx quoting convention.
+
+        Parameters
+        ---------
+        quote_ccy: str
+            Quote currency
+
+        Returns
+        -------
+        quote_ccy: str
+            Quote currency.
+        """
+        mkts = []  # fx pairs list
+
+        # fx groups
+        base_ccys = ["EUR", "GBP", "AUD", "NZD"]
+        g10_fx = ['USD', 'EUR', 'GBP', 'JPY', 'CHF', 'CAD', 'AUD', 'NZD', 'NOK', 'SEK']
+        dm_fx = ['USD', 'EUR', 'GBP', 'JPY', 'CHF', 'CAD', 'AUD', 'NZD', 'NOK', 'SEK', 'DKK', 'SGD', 'HKD']
+        em_fx = ['ARS', 'BRL', 'CHN', 'CLP', 'CNY', 'COP', 'CZK', 'HUF', 'IDR', 'INR', 'ILS', 'KRW', 'MYR', 'MXN',
+        'PEN', 'PHP', 'PLN', 'RUB', 'THB', 'TRY', 'TWD', 'ZAR']
+        em_ndf_fx = ['ARS', 'BRL', 'CNY', 'CLP', 'COP', 'IDR', 'INR', 'KRW', 'PEN', 'PHP', 'RUB', 'THB', 'TRY', 'TWD']
+
+        for ticker in self.data_req.tickers:
+            if ticker.upper() in base_ccys and quote_ccy.upper() == "USD":
+                mkts.append(ticker.upper() + "/" + quote_ccy.upper())
+            elif quote_ccy.upper() == "USD":
+                mkts.append(quote_ccy.upper() + "/" + ticker.upper())
+            else:
+                mkts.append(ticker.upper() + "/" + quote_ccy.upper())
+
+        return mkts
+
+    def convert_fields(self, data_source: str) -> List[str]:
+        """
+        Converts fields from CryptoDataPy to data source format.
+
+        Parameters
+        ---------
+        data_source: str
+            Name of data source for fields conversions.
+
+        Returns
+        -------
+        fields_list: list
+            List of fields in data source format.
+
+        """
+        # fields
+        with resources.path("cryptodatapy.conf", "fields.csv") as f:
+            fields_dict_path = f
+        fields_df, fields_list = (
+            pd.read_csv(fields_dict_path, index_col=0, encoding="latin1"),
+            [],
+        )
+
+        # when source fields already provided in data req
+        if self.data_req.source_fields is not None:
+            fields_list = self.data_req.source_fields
+
+        # convert to source format
+        else:
+            for field in self.data_req.fields:
+                try:
+                    fields_list.append(fields_df.loc[field, data_source + "_id"])
+                except KeyError as e:
+                    logging.warning(e)
+                    logging.warning(
+                        f"Id for {field} could not be found in the data catalog."
+                        f" Try using source field ids."
+                    )
+
+        return fields_list
