@@ -510,13 +510,13 @@ class WrangleData:
         self.filter_dates()
         # resample
         self.data_resp = self.data_resp.resample(self.data_req.freq).last()
-        # type conversion
-        self.data_resp = self.data_resp.apply(pd.to_numeric, errors='coerce').convert_dtypes()
         # remove bad data
         self.data_resp = self.data_resp[self.data_resp != 0]  # 0 values
         # filter dups and NaNs
         self.data_resp = self.data_resp[~self.data_resp.index.duplicated()]  # duplicate rows
         self.data_resp = self.data_resp.dropna(how='all').dropna(how='all', axis=1)  # entire row or col NaNs
+        # type conversion
+        self.data_resp = self.data_resp.convert_dtypes()
 
         return self.data_resp
 
@@ -528,7 +528,6 @@ class WrangleData:
         -------
         pd.DataFrame
             Wrangled dataframe into tidy data format.
-
         """
         # convert fields to lib
         self.convert_fields_to_lib(data_source='coinmetrics')
@@ -640,33 +639,78 @@ class WrangleData:
 
         """
         # create df
-        if data_type == 'eqty' or data_type == 'crypto':
+        if data_type == 'crypto':
             self.data_resp = pd.DataFrame(self.data_resp[0]['priceData'])
         else:
             self.data_resp = pd.DataFrame(self.data_resp)
+
         # convert fields to lib
         self.convert_fields_to_lib(data_source='tiingo')
+
         # convert to datetime
         self.data_resp['date'] = pd.to_datetime(self.data_resp['date'])
+
         # set index
         self.data_resp = self.data_resp.set_index('date').sort_index()
         self.data_resp.index = self.data_resp.index.tz_localize(None)
+
         # resample
         self.data_resp = self.data_resp.resample(self.data_req.freq).last()
+
         # reformat index
         if self.data_req.freq in ['d', 'w', 'm', 'q']:
             self.data_resp.reset_index(inplace=True)
             self.data_resp.date = pd.to_datetime(self.data_resp.date.dt.date)
             # reset index
             self.data_resp.set_index('date', inplace=True)
+
         # type conversion
-        self.data_resp = self.data_resp.apply(pd.to_numeric, errors='coerce').convert_dtypes()
+        self.data_resp = self.data_resp.convert_dtypes()
+
         # remove bad data
         self.data_resp = self.data_resp[~self.data_resp.index.duplicated()]  # duplicate rows
         self.data_resp = self.data_resp.dropna(how='all').dropna(how='all', axis=1)  # entire row or col NaNs
 
         return self.data_resp
 
+    def polygon(self) -> pd.DataFrame:
+        """
+        Wrangles Polygon data response to dataframe with tidy data format.
+
+        Returns
+        -------
+        pd.DataFrame
+            Wrangled dataframe into tidy data format.
+
+        """
+        # create df
+        self.data_resp = pd.DataFrame(self.data_resp)
+
+        # convert cols/fields to lib
+        self.convert_fields_to_lib(data_source='polygon')
+
+        # convert to datetime
+        self.data_resp['date'] = pd.to_datetime(self.data_resp['date'], unit='ms')
+
+        # set index
+        self.data_resp = self.data_resp.set_index('date').sort_index()
+
+        # resample
+        self.data_resp = self.data_resp.resample(self.data_req.freq).last()
+
+        # type conversion
+        self.data_resp = self.data_resp.convert_dtypes()
+
+        # remove bad data
+        self.data_resp = self.data_resp[~self.data_resp.index.duplicated()]  # duplicate rows
+        self.data_resp = self.data_resp.dropna(how='all').dropna(how='all', axis=1)  # entire row or col NaNs
+        self.data_resp = self.data_resp[self.data_resp != 0]
+
+        # keep only requested fields and sort index
+        self.data_resp = self.data_resp[self.data_req.fields].sort_index()
+
+        return self.data_resp
+
     def investpy(self) -> pd.DataFrame:
         """
         Wrangles InvestPy data response to dataframe with tidy data format.
@@ -795,10 +839,10 @@ class WrangleData:
         # resample
         if self.data_req.freq in ['d', 'w', 'm', 'q', 'y']:
             self.tidy_data = (
-                    (1 + self.tidy_data.funding_rate)
+                    self.tidy_data.funding_rate
                     .groupby('ticker')
                     .resample('d', level='date')
-                    .prod() - 1
+                    .sum()
             ).to_frame().swaplevel('ticker', 'date').sort_index()
 
         return self.tidy_data
@@ -813,10 +857,7 @@ class WrangleData:
             Dataframe with tidy data format.
         """
         # add tickers
-        for i in range(len(self.data_req.source_markets)):
-            df = pd.DataFrame(self.data_resp[i])
-            self.tidy_data = pd.concat([self.tidy_data, df])
-        self.tidy_data = self.tidy_data[['symbol', 'openInterestAmount', 'datetime']]
+        self.tidy_data = pd.DataFrame(self.data_resp)[['symbol', 'openInterestAmount', 'datetime']]
         self.data_resp = self.tidy_data
 
         # convert to lib fields
@@ -874,8 +915,17 @@ class WrangleData:
         self.data_resp.columns = self.data_req.tickers  # convert tickers to cryptodatapy format
 
         # resample to match end of reporting period, not beginning
-        self.data_resp = self.data_resp.resample('d').last().ffill().resample(self.data_req.freq).last().stack(). \
-            to_frame().reset_index()
+        self.data_resp = (
+            self.data_resp
+            .resample('d')
+            .last()
+            .ffill()
+            .resample(self.data_req.freq)
+            .last()
+            .stack(future_stack=True)
+            .to_frame()
+            .reset_index()
+        )
 
         # convert cols
         if self.data_req.cat == 'macro':
@@ -930,7 +980,39 @@ class WrangleData:
             last().swaplevel('ticker', 'date').sort_index()
 
         # type conversion
-        self.data_resp = self.data_resp.apply(pd.to_numeric, errors='coerce').convert_dtypes()
+        self.data_resp = self.data_resp.convert_dtypes()
+
+        # remove bad data
+        self.data_resp = self.data_resp[self.data_resp != 0]  # 0 values
+        self.data_resp = self.data_resp[~self.data_resp.index.duplicated()]  # duplicate rows
+        self.data_resp = self.data_resp.dropna(how='all').dropna(how='all', axis=1)  # entire row or col NaNs
+
+        # keep only requested fields and sort index
+        self.data_resp = self.data_resp[self.data_req.fields].sort_index()
+
+        return self.data_resp
+
+    def alphavantage(self) -> pd.DataFrame:
+        """
+        Wrangles Alpha Vantage data response to dataframe with tidy data format.
+
+        Returns
+        -------
+        pd.DataFrame
+            Wrangled dataframe into tidy data format.
+        """
+        # index
+        self.data_resp.reset_index(inplace=True)
+        self.data_resp['date'] = pd.to_datetime(self.data_resp['date'])
+        self.data_resp.set_index(['date', 'ticker'], inplace=True)
+
+        # resample
+        self.data_resp = self.data_resp.groupby('ticker').\
+            resample(self.data_req.freq, level='date').\
+            last().swaplevel('ticker', 'date').sort_index()
+
+        # type conversion
+        self.data_resp = self.data_resp.convert_dtypes()
 
         # remove bad data
         self.data_resp = self.data_resp[self.data_resp != 0]  # 0 values
@@ -976,7 +1058,7 @@ class WrangleData:
 
         # format index
         self.data_resp.index.name = 'date'  # rename
-        self.data_resp = self.data_resp.stack().to_frame('er')
+        self.data_resp = self.data_resp.stack(future_stack=True).to_frame('er')
         self.data_resp.index.names = ['date', 'ticker']
 
         # type and conversion to decimals
@@ -1003,7 +1085,7 @@ class WrangleData:
         with resources.path("cryptodatapy.conf", "tickers.csv") as f:
             tickers_path = f
         tickers_df = pd.read_csv(tickers_path, index_col=0, encoding="latin1")
-        self.data_resp = self.data_resp.stack().to_frame()  # stack df
+        self.data_resp = self.data_resp.stack(future_stack=True).to_frame()  # stack df
         # create list of tickers using tickers csv
         tickers = []
         for row in self.data_resp.iterrows():
@@ -1085,11 +1167,11 @@ class WrangleData:
         self.data_resp = df
         self.filter_dates()
         # stack df
-        self.data_resp = self.data_resp.stack().to_frame('er')
+        self.data_resp = self.data_resp.stack(future_stack=True).to_frame('er')
         # create multi index
         self.data_resp.index.names = ['date', 'ticker']
         # type and conversion to decimals
-        self.data_resp = self.data_resp.apply(pd.to_numeric, errors='coerce').convert_dtypes()
+        self.data_resp = self.data_resp.convert_dtypes()
         # remove bad data
         self.data_resp = self.data_resp[~self.data_resp.index.duplicated()]  # duplicate rows
         self.data_resp = self.data_resp.dropna(how='all').dropna(how='all', axis=1)  # entire row or col NaNs
